//=============================================================================
// MutOLGameStats
//=============================================================================
class MutOLGameStats extends Mutator
    config(OLStats);

var class<OLGameStats> StatClass;

var config bool bExportRawLog;
var config bool bExportHTMLLogs;
var config bool bLogBots;
var config bool bLogChat;
var config bool bLogPickups;
var config bool bLogVehicleDeaths;
var config bool bLogSecurityInfo;
var config bool bLogPings;
var config bool bLogAccuracy;
var config bool bAdvancedVoteLogging;
var config bool bMasterServerStats;
var localized string LogBotsDisplayText, LogPickupsDisplayText, LogAccuracyDisplayText;
var localized string LogBotsDescription, LogPickupsDescription, LogAccuracyDescription;
var localized string MasterStatsDisplayText, ExportRawLogDisplayText, ExportHTMLLogDisplayText, KeepBotRecordsDisplayText;
var localized string MasterStatsDescription, ExportRawLogDescription, ExportHTMLLogDescription, KeepBotRecordsDescription;

var config bool bRunUTStatsDBParser;
var config string ParserURL;
var config string ParserPass;
var config bool bUploadLogs;
var config string UploadURL;
var config string UploadPass;

var config int MinPlayers;
var config int MinHumans;
var config bool KeepBotRecords;

var OLLocalGameStats GS;

// This mutator can get spawned in 2 ways.
// The player can add it to his mutator list. If this is the case, then this
// mutator spawns StatClass.
// The other way is that the GameStatsClass was set to OLStats in the server ini, therefore
// OLLocalGameStats was already spawned, and IT spawned this actor.
function PreBeginPlay()
{
    local OLGameStats OLGS;

    foreach AllActors(class'OLLocalGameStats', GS)
    {
        break;
    }

    if(GS == none)
    { // This was added as a mutator, and therefore it spawns the stats
        if(Level.NetMode==NM_DedicatedServer)
        {
            Log("ERROR: This is a dedicated server. Dedicated servers should not use the OLStats mutator. See readme.",'OLStats');
            return;
        }
        level.game.bEnableStatLogging = true;
        level.game.bLoggingGame = true;
        OLGS = spawn(StatClass);
        level.game.GameStats = OLGS;
    }
    else
    { // The stats spawned this actor.
    }
}

function bool CanEnterVehicle(Vehicle V, Pawn P)
{
    local bool bCanEnter;

    if( NextMutator != None )
        bCanEnter = NextMutator.CanEnterVehicle(V, P);
    else
        bCanEnter = true;

    if (GS != none && bCanEnter && !V.bTeamLocked && (P.GetTeamNum() != V.Team))
        GS.GameEvent( "carjack", GetItemName(string(V)),  P.PlayerReplicationInfo );

    return bCanEnter;
}

static function FillPlayInfo(PlayInfo PlayInfo)
{
    Super.FillPlayInfo(PlayInfo);

    PlayInfo.AddSetting(default.RulesGroup, "bExportRawLog", default.ExportRawLogDisplayText, 0, 1, "Check", , , , false);
    PlayInfo.AddSetting(default.RulesGroup, "bExportHTMLLogs", default.ExportHTMLLogDisplayText, 0, 1, "Check", , , , false);
    PlayInfo.AddSetting(default.RulesGroup, "KeepBotRecords", default.KeepBotRecordsDisplayText, 0, 1, "Check", , , , false);
    PlayInfo.AddSetting(default.RulesGroup, "bLogBots", default.LogBotsDisplayText, 0, 1, "Check", , , , false);
    PlayInfo.AddSetting(default.RulesGroup, "bLogPickups", default.LogPickupsDisplayText, 0, 1, "Check", , , , true);
    PlayInfo.AddSetting(default.RulesGroup, "bLogAccuracy", default.LogAccuracyDisplayText, 0, 1, "Check", , , , true);
    PlayInfo.AddSetting(default.RulesGroup, "bMasterServerStats", default.MasterStatsDisplayText, 0, 1, "Check", , , , true);
}

static event string GetDescriptionText(string PropName)
{
    switch (PropName)
    {
        case "bExportRawLog":           return default.ExportRawLogDescription;
        case "bExportHTMLLogs":         return default.ExportHTMLLogDescription;
        case "KeepBotRecords":          return default.KeepBotRecordsDescription;
        case "bLogBots":                return default.LogBotsDescription;
        case "bLogPickups":             return default.LogPickupsDescription;
        case "bLogAccuracy":            return default.LogAccuracyDescription;
        case "bMasterServerStats":      return default.MasterStatsDescription;
    }

    return Super.GetDescriptionText(PropName);
}

defaultproperties
{
    StatClass=class'OLStats.OLGameStats'
    FriendlyName="OLStats"
    Description="Turns on stat logging for listen servers and Instant Action games."

    bExportRawLog=true
    ExportRawLogDisplayText="Export Raw Logs"
    ExportRawLogDescription="Enables exporting of the raw log files to your UserLogs directory."
    bExportHTMLLogs=true
    ExportHTMLLogDisplayText="Export HTML Logs"
    ExportHTMLLogDescription="This enables the \"Local OLStats Database\" which outputs local HTML logs of all your matches."
    KeepBotRecords=false
    KeepBotRecordsDisplayText="Keep Bot Career Records"
    KeepBotRecordsDescription="If true, bots will be included on the players page, as well as have a chance to make it in to career and match highs."
    bLogBots=true
    LogBotsDisplayText="Log Bots"
    LogBotsDescription="Include bots in the log."
    bLogPickups=true
    LogPickupsDisplayText="Log Pickups"
    LogPickupsDescription="Log every item and weapon that is picked up. Only affects raw logging."
    bLogAccuracy=true
    LogAccuracyDisplayText="Log Accuracy"
    LogAccuracyDescription="Log the number of shots fired and the number of shots hit for every weapon of each player. Only affects raw logging."
    bMasterServerStats=true
    MasterStatsDisplayText="Enable global Stats"
    MasterStatsDescription="Uploads stats to the official Epic master stats server."

    bLogVehicleDeaths=true
    bLogSecurityInfo=true
    bLogPings=true
    bAdvancedVoteLogging=false
    bLogChat=true

    bRunUTStatsDBParser=false
    ParserURL="http://myserver.com/logs.php"
    ParserPass="parsepass"
    bUploadLogs=false
    UploadURL="http://myserver.com/OLSendLog.php"
    UploadPass="adminpass"

    MinHumans = 1
    MinPlayers = 2
}
