//=============================================================================
// OLStats.OLGameStatRules
//=============================================================================
class OLGameStatRules extends GameRules;

struct ScoreCacheStruct
{
    var PlayerReplicationInfo PRI;
    var float Score;
};

var array<ScoreCacheStruct> ScoreCache;
var OLLocalGameStats GS;

function bool OverridePickupQuery(Pawn Other, Pickup item, out byte bAllowPickup)
{
    // Quick hack to make weapons not register as multiple pickups with weaponstay on
    if (GS != none && (!Level.Game.bWeaponStay || !PlayerHasWeapon(Other,item) ) )
        GS.LogPickup(Other.PlayerReplicationInfo, item);
    return Super.OverridePickupQuery(Other, item, bAllowPickup);
}

function bool PlayerHasWeapon(Pawn Other, Pickup item)
{
    local inventory inv;
    local inventory rpginv;

    if( WeaponPickup(item) == none )
    {
        return false;
    }

    for(inv=Other.inventory;inv!=none;inv=inv.inventory)
    {
        if (Weapon(inv) != none && GS.RPGHelper != none)
            rpginv = GS.RPGHelper.RPGGetRealWeapon( Weapon(inv) );

        if (inv.class == item.InventoryType || (rpginv != none && rpginv.class == item.InventoryType))
            return true;
    }
    return false;
}

function ScoreKill(Controller Killer, Controller Killed)
{
    local xMutantGame MutantGame;
    local UnrealPlayer uKiller;
    local int KillScore;


    // SPECIAL MUTANT HANDLING
    // Mutant scores are reported all wrong, so I manually do it here.
    // This is basically a copy of all the code from xMutantGame::ScoreKill,
    // modified to only send score events to the OLGameStats.
    if ( xMutantGame(Level.Game) != none && GS != none)
    {
        MutantGame = xMutantGame(Level.Game);

        if (Killer == Killed || Killer == none)
        {
            if(Killed == MutantGame.CurrentMutant)
                GS.MutantScoreEvent(Killed.PlayerReplicationInfo,-1 * MutantGame.MutantSuicidePenalty,"mutant_self_kill");
            else
                GS.MutantScoreEvent(Killed.PlayerReplicationInfo,-1 * MutantGame.BaseKillScore,"self_kill");
        }
        else if ( killer.PlayerReplicationInfo != None )
        {
            if (MutantGame.CurrentMutant != none) // Mutant present
            {
                if(Killed == MutantGame.CurrentMutant) // Mutant was killed
                {
                    if ( Killed.PlayerReplicationInfo != None )
                        KillScore = 0.5 * (Killed.PlayerReplicationInfo.Score - GetCachedScore(Killer.PlayerReplicationInfo));
                    KillScore = Max(KillScore, MutantGame.BaseKillScore);

                    GS.MutantScoreEvent(Killer.PlayerReplicationInfo,KillScore,"frag");
                }
                else if(Killer == MutantGame.CurrentMutant) // Mutant killed someone
                {
                    uKiller = UnrealPlayer(Killer);

                    if(uKiller != None)
                    {
                        // If the mutant killed the bottom feeder, they get a special score
                        // Otherwise - score base on multi-kill level

                        if( MutantGame.IsBottomFeeder(Killed) )
                            GS.MutantScoreEvent(Killer.PlayerReplicationInfo,MutantGame.BottomFeederKillScore,"frag");
                        else
                            GS.MutantScoreEvent(Killer.PlayerReplicationInfo,Min( MutantGame.BaseKillScore + (uKiller.MultiKillLevel * MutantGame.MultiKillBonus), MutantGame.MaxMultiKillScore ),"frag");
                    }
                    else
                    {
                        GS.MutantScoreEvent(Killer.PlayerReplicationInfo,MutantGame.BaseKillScore,"frag");
                    }
                }
                else if( MutantGame.IsBottomFeeder(Killer) )
                {
                    GS.MutantScoreEvent(Killer.PlayerReplicationInfo,MutantGame.BaseKillScore,"bottom_feeder_frag");
                }
            }
            else if (MutantGame.CurrentMutant == none) // No Mutant
            {
                GS.MutantScoreEvent(Killer.PlayerReplicationInfo,MutantGame.BaseKillScore,"frag");
            }
        }
    }

    if ( NextGameRules != None )
        NextGameRules.ScoreKill(Killer,Killed);
}

function bool PreventDeath(Pawn Killed, Controller Killer, class<DamageType> damageType, vector HitLocation)
{
    if ( (NextGameRules != None) && NextGameRules.PreventDeath(Killed,Killer, damageType,HitLocation) )
        return true;

    if (GS == none)
        return false;

    if ( Killed != none && MonsterController(Killed.Controller) != none  && Invasion(Level.Game) != none)
        GS.MonsterDeath(Killer.PlayerReplicationInfo, Killed, damageType);

    // If a monster was the killer, the victim exists, and the victim is not another monster
    if ( MonsterController(Killer) != none && Killed != none && Killed.Controller != none && MonsterController(Killed.Controller) == none && Invasion(Level.Game) != none)
        GS.MonsterKill(Killer.Pawn, Killed.Controller.PlayerReplicationInfo);

    // If this is invasion, call our special invasion kill event
    if ( Invasion(Level.Game) != none && MonsterController(Killer) == none && Killed != none && MonsterController(Killed.Controller) == none)
        if (Killer != none)
            GS.InvasionKillEvent( Killer.PlayerReplicationInfo, Killed.Controller.PlayerReplicationInfo, damageType );
        else
            GS.InvasionKillEvent( none, Killed.Controller.PlayerReplicationInfo, damageType );


    return false;
}

// This is passed to the Gamestats, and then it passes it to each Accuracy Handler to calculate accuracy.
function int NetDamage( int OriginalDamage, int Damage, pawn injured, pawn instigatedBy, vector HitLocation, out vector Momentum, class<DamageType> DamageType )
{
    local int NewDamage;

    if ( NextGameRules != None )
        NewDamage = NextGameRules.NetDamage( OriginalDamage,Damage,injured,instigatedBy,HitLocation,Momentum,DamageType );
    else
        NewDamage = Damage;

    if (GS != none && instigatedBy != none && instigatedBy.controller != none && instigatedBy.Controller.PlayerReplicationInfo != none)
        GS.DamageDealt(instigatedBy.Controller.PlayerReplicationInfo, DamageType, NewDamage);

    return NewDamage;
}

function float GetCachedScore(PlayerReplicationInfo PRI)
{
    local int i;

    for(i=0;i<ScoreCache.Length;i++)
    {
        if(ScoreCache[i].PRI == PRI)
            return ScoreCache[i].Score;
    }

    return 0;
}

function Tick(float Delta)
{
    local int i, x;
    local GameReplicationInfo GRI;

    ScoreCache.Length = 0;

    GRI = Level.Game.GameReplicationInfo;

    for(i=0;i<GRI.PRIArray.Length;i++)
    {
        x = ScoreCache.Length;
        ScoreCache.Length = x + 1;
        ScoreCache[x].PRI = GRI.PRIArray[i];
        ScoreCache[x].Score = GRI.PRIArray[i].Score;
    }
}

defaultproperties
{
}
