//-----------------------------------------------------------
//
//-----------------------------------------------------------
class OLGameStats extends GameStats;

var OLMasterServerGameStats MasterStats;
var OLLocalGameStats LocalStats;

event PreBeginPlay()
{
    Super.PreBeginPlay();

    if (class'MutOLGameStats'.default.bMasterServerStats)
        MasterStats = spawn(class'OLMasterServerGameStats');

    LocalStats = spawn(class'OLLocalGameStats');
}

function Init()
{
}

function Shutdown()
{
    if (MasterStats != none)
        MasterStats.Destroy();

    if (LocalStats != none)
        LocalStats.Destroy();
}

function Logf(string LogString)
{
    // The only reason this would get called is if something wanted to manually
    // add a line to the stat log.  I'll pass it on just in case its important.

    if (MasterStats != none)
        MasterStats.Logf(LogString);

    if (LocalStats != none)
        LocalStats.Logf(LogString);
}



function NewGame()
{
    if (MasterStats != none)
        MasterStats.NewGame();

    if (LocalStats != none)
        LocalStats.NewGame();
}

function ServerInfo()
{
    if (MasterStats != none)
        MasterStats.ServerInfo();

    if (LocalStats != none)
        LocalStats.ServerInfo();
}

function StartGame()
{
    if (MasterStats != none)
        MasterStats.StartGame();

    if (LocalStats != none)
        LocalStats.StartGame();
}


// Send stats for the end of the game
function EndGame(string Reason)
{
    if (MasterStats != none)
        MasterStats.EndGame(Reason);

    if (LocalStats != none)
        LocalStats.EndGame(Reason);
}


// Connect Events get fired every time a player connects to a server
function ConnectEvent(PlayerReplicationInfo Who)
{
    if (MasterStats != none)
        MasterStats.ConnectEvent(Who);

    if (LocalStats != none)
        LocalStats.ConnectEvent(Who);
}

// Connect Events get fired every time a player connects or leaves from a server
function DisconnectEvent(PlayerReplicationInfo Who)
{
    if (MasterStats != none)
        MasterStats.DisconnectEvent(Who);

    if (LocalStats != none)
        LocalStats.DisconnectEvent(Who);
}


// Scoring Events occur when a player's score changes
function ScoreEvent(PlayerReplicationInfo Who, float Points, string Desc)
{
    if (MasterStats != none)
        MasterStats.ScoreEvent(Who, Points, Desc);

    if (LocalStats != none)
        LocalStats.ScoreEvent(Who, Points, Desc);
}


function TeamScoreEvent(int Team, float Points, string Desc)
{
    if (MasterStats != none)
        MasterStats.TeamScoreEvent(Team, Points, Desc);

    if (LocalStats != none)
        LocalStats.TeamScoreEvent(Team, Points, Desc);
}


// KillEvents occur when a player kills, is killed, suicides
function KillEvent(string Killtype, PlayerReplicationInfo Killer, PlayerReplicationInfo Victim, class<DamageType> Damage)
{
    if (MasterStats != none)
        MasterStats.KillEvent(Killtype, Killer, Victim, Damage);

    if (LocalStats != none)
        LocalStats.KillEvent(Killtype, Killer, Victim, Damage);
}


// Special Events are everything else regarding the player
function SpecialEvent(PlayerReplicationInfo Who, string Desc)
{
    if (MasterStats != none)
        MasterStats.SpecialEvent(Who, Desc);

    if (LocalStats != none)
        LocalStats.SpecialEvent(Who, Desc);
}


// Special events regarding the game
function GameEvent(string GEvent, string Desc, PlayerReplicationInfo Who)
{
    if (MasterStats != none)
        MasterStats.GameEvent(GEvent, Desc, Who);

    if (LocalStats != none)
        LocalStats.GameEvent(GEvent, Desc, Who);
}




DefaultProperties
{

}
