/*******************************************************************************
    OLStatsAccuracyCounter

    Creation date: 02/04/2004 10:02
    Copyright (c) 2004, Greg Laabs
    <!-- $Id$ -->
*******************************************************************************/

class OLStatsAccuracyCounter extends Info;

struct export WeaponAccuracy
{
    var Class<Weapon> WeaponClass;
    var Weapon Weapon;
    var string WeaponName;
    var int ShotsFired;
    var int ShotsHit;
    var float LastHitTime;
    var int TotalDamage;
    var float RefireRate;
};

var PlayerReplicationInfo PRI;
var Array<WeaponAccuracy> WeaponAccuracyArray;
var Weapon LastWeapon;
var int LastAmmoCount;
var int LastWeaponArrayPointer;
var OLLocalGameStats GS;

function Tick(float Delta)
{
    local weapon CurrentWeapon;
    local int i;
    local bool bFound;
    local string LastWeaponName;


    if( PRI == None || Controller(PRI.Owner).Pawn == None || Controller(PRI.Owner).Pawn.Weapon == none)
        return;

    CurrentWeapon = Controller(PRI.Owner).Pawn.Weapon;

    if (GS.RPGHelper != none)
        CurrentWeapon = GS.RPGHelper.RPGGetRealWeapon(CurrentWeapon);

    if (CurrentWeapon == LastWeapon)
    {
        if ( CurrentWeapon.AmmoAmount(0) < LastAmmoCount )
        {
            WeaponAccuracyArray[LastWeaponArrayPointer].ShotsFired++;
//            PlayerController(PRI.Owner).ClientMessage("You fired your "$WeaponAccuracyArray[LastWeaponArrayPointer].WeaponName$"! Shots fired: "$WeaponAccuracyArray[LastWeaponArrayPointer].ShotsFired);
        }
        LastAmmoCount = CurrentWeapon.AmmoAmount(0);


    } else { // Player changed his weapon
        LastWeapon = CurrentWeapon;
        LastAmmoCount = CurrentWeapon.AmmoAmount(0);

        LastWeaponName = GetItemName(String(LastWeapon));
        if (LastWeaponName == "RPGLinkGun")
            LastWeaponName = "LinkGun";

        bFound = false;
        for(i=0;i<WeaponAccuracyArray.Length;i++) // Check to see if the weapon is already in the array
        {
            if ( LastWeaponName == WeaponAccuracyArray[i].WeaponName ) // If it is in the array, set our pointer to it.
            {
                LastWeaponArrayPointer = i;
                bFound = true;
                continue;
            }
        }

        if (!bFound) // This is the first time switching to this weapon: Add it to the array
        {
            i = WeaponAccuracyArray.Length;
            WeaponAccuracyArray.Length = i + 1;
            WeaponAccuracyArray[i].WeaponName = LastWeaponName;
            WeaponAccuracyArray[i].WeaponClass = LastWeapon.Class;
            WeaponAccuracyArray[i].ShotsFired = 0;
            WeaponAccuracyArray[i].ShotsHit = 0;
            WeaponAccuracyArray[i].LastHitTime = 0;
            WeaponAccuracyArray[i].TotalDamage = 0;
            WeaponAccuracyArray[i].RefireRate = LastWeapon.GetFireMode(0).FireRate;
            LastWeaponArrayPointer = i;
//            PlayerController(PRI.Owner).ClientMessage("You switched to a new weapon: "$WeaponAccuracyArray[i].WeaponName);
        }
    }
}

function DamageDealt(class <DamageType> DamageType, int Damage)
{
    local string WeaponClass;
    local int i;
    local int ArrayPointer;
    local bool bFound;
    local float TimeSinceLastHit;

    WeaponClass = GetItemName(DamageType.static.GetWeaponClass());

    // First check to see if it's the current weapon.  This should save processing power so it doesn't have
    // to go through the for loop every time.
    if ( WeaponAccuracyArray[LastWeaponArrayPointer].WeaponName == WeaponClass )
    {
        ArrayPointer=LastWeaponArrayPointer;
    } else {
        for(i=0;i<WeaponAccuracyArray.Length;i++) // Find the weapon accuracy struct that this damagetype goes to.
        {
            if( WeaponAccuracyArray[i].WeaponName == WeaponClass )
            {
                ArrayPointer = i;
                bFound = true;
                continue;
            }
        }
        if (!bFound)
        {
//            Log("Could not match the damagetype to a weapon! DamType="$DamageType$", WeaponClass="$WeaponClass);
            return;
        }
    }

    TimeSinceLastHit = Level.TimeSeconds - WeaponAccuracyArray[ArrayPointer].LastHitTime;

    if( !DamageType.default.bInstantHit && TimeSinceLastHit < 0.5 * WeaponAccuracyArray[ArrayPointer].RefireRate )
    {
//        PlayerController(PRI.Owner).ClientMessage("Too fast! Fire Rate: "$ WeaponAccuracyArray[ArrayPointer].RefireRate );
        return;
    }

    WeaponAccuracyArray[ArrayPointer].ShotsHit++;
    WeaponAccuracyArray[ArrayPointer].TotalDamage += Damage;
    WeaponAccuracyArray[ArrayPointer].LastHitTime = Level.TimeSeconds;
//    PlayerController(PRI.Owner).ClientMessage("You hit something with your "$WeaponClass$".  Shots Fired: "$WeaponAccuracyArray[ArrayPointer].ShotsFired$" Total hits: "$WeaponAccuracyArray[ArrayPointer].ShotsHit$" Accuracy:"$ (WeaponAccuracyArray[ArrayPointer].ShotsHit * 1.f) / (WeaponAccuracyArray[LastWeaponArrayPointer].ShotsFired * 1.f)*100.f $"%" );
}

defaultproperties
{
}
