//-----------------------------------------------------------
//
//-----------------------------------------------------------
class OLStatsVotingHandler extends xVotingHandler;

var OLLocalGameStats GS;

function SubmitMapVote(int MapIndex, int GameIndex, Actor Voter)
{
    local int Index, VoteCount, PrevMapVote, PrevGameVote;
    local MapHistoryInfo MapInfo;

    if(bLevelSwitchPending)
        return;

    Index = GetMVRIIndex(PlayerController(Voter));

    if(PlayerController(Voter).PlayerReplicationInfo.bAdmin)  // Administrator Vote
    {
        TextMessage = lmsgAdminMapChange;
        TextMessage = Repl(TextMessage, "%mapname%", MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")");
        Level.Game.Broadcast(self,TextMessage);

        log("Admin has forced map switch to " $ MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")",'MapVote');

        if (GS != none)
        {
            GS.LogMapVote(Controller(Voter).PlayerReplicationInfo,MapList[MapIndex].MapName,VoteCount);
            GS.GameEvent("mapvote",MapList[MapIndex].MapName,Controller(Voter).PlayerReplicationInfo);
        }

        CloseAllVoteWindows();

        bLevelSwitchPending = true;

        MapInfo = History.PlayMap(MapList[MapIndex].MapName);

        ServerTravelString = SetupGameMap(MapList[MapIndex], GameIndex, MapInfo);
        log("ServerTravelString = " $ ServerTravelString ,'MapVoteDebug');

        Level.ServerTravel(ServerTravelString, false);    // change the map

        settimer(1,true);
        return;
    }

    if( PlayerController(Voter).PlayerReplicationInfo.bOnlySpectator )
    {
        // Spectators cant vote
        PlayerController(Voter).ClientMessage(lmsgSpectatorsCantVote);
        return;
    }

    // check for invalid map, invalid gametype, player isnt revoting same as previous vote, and map choosen isnt disabled
    if(MapIndex < 0 ||
        MapIndex >= MapCount ||
        GameIndex >= GameConfig.Length ||
        (MVRI[Index].GameVote == GameIndex && MVRI[Index].MapVote == MapIndex) ||
        !MapList[MapIndex].bEnabled)
        return;

    log("___" $ Index $ " - " $ PlayerController(Voter).PlayerReplicationInfo.PlayerName $ " voted for " $ MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")",'MapVote');

    PrevMapVote = MVRI[Index].MapVote;
    PrevGameVote = MVRI[Index].GameVote;
    MVRI[Index].MapVote = MapIndex;
    MVRI[Index].GameVote = GameIndex;

    if(bAccumulationMode)
    {
        if(bScoreMode)
        {
            VoteCount = GetAccVote(PlayerController(Voter)) + int(GetPlayerScore(PlayerController(Voter)));
            TextMessage = lmsgMapVotedForWithCount;
            TextMessage = repl(TextMessage, "%playername%", PlayerController(Voter).PlayerReplicationInfo.PlayerName );
            TextMessage = repl(TextMessage, "%votecount%", string(VoteCount) );
            TextMessage = repl(TextMessage, "%mapname%", MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")" );
            Level.Game.Broadcast(self,TextMessage);
        }
        else
        {
            VoteCount = GetAccVote(PlayerController(Voter)) + 1;
            TextMessage = lmsgMapVotedForWithCount;
            TextMessage = repl(TextMessage, "%playername%", PlayerController(Voter).PlayerReplicationInfo.PlayerName );
            TextMessage = repl(TextMessage, "%votecount%", string(VoteCount) );
            TextMessage = repl(TextMessage, "%mapname%", MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")" );
            Level.Game.Broadcast(self,TextMessage);
        }
    }
    else
    {
        if(bScoreMode)
        {
            VoteCount = int(GetPlayerScore(PlayerController(Voter)));
            TextMessage = lmsgMapVotedForWithCount;
            TextMessage = repl(TextMessage, "%playername%", PlayerController(Voter).PlayerReplicationInfo.PlayerName );
            TextMessage = repl(TextMessage, "%votecount%", string(VoteCount) );
            TextMessage = repl(TextMessage, "%mapname%", MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")" );
            Level.Game.Broadcast(self,TextMessage);
        }
        else
        {
            VoteCount =  1;
            TextMessage = lmsgMapVotedFor;
            TextMessage = repl(TextMessage, "%playername%", PlayerController(Voter).PlayerReplicationInfo.PlayerName );
            TextMessage = repl(TextMessage, "%mapname%", MapList[MapIndex].MapName $ "(" $ GameConfig[GameIndex].Acronym $ ")" );
            Level.Game.Broadcast(self,TextMessage);
        }
    }

    if (GS != none)
        GS.LogMapVote(Controller(Voter).PlayerReplicationInfo,MapList[MapIndex].MapName,VoteCount);

    UpdateVoteCount(MapIndex, GameIndex, VoteCount);
    if( PrevMapVote > -1 && PrevGameVote > -1 )
        UpdateVoteCount(PrevMapVote, PrevGameVote, -MVRI[Index].VoteCount); // undo previous vote
    MVRI[Index].VoteCount = VoteCount;
    TallyVotes(false);
}

function SubmitKickVote(int PlayerID, Actor Voter)
{
    local int VoterID, VictumID, i, PreviousVote;
    local bool bFound;
    local string PlayerName;

    log("SubmitKickVote " $ PlayerID, 'MapVoteDebug');

    if(bLevelSwitchPending || !bKickVote)
        return;

    VoterID = GetMVRIIndex(PlayerController(Voter));

    // Find Player
    bFound = false;
    for(i=0;i < MVRI.Length;i++)
    {
        if(MVRI[i] != none && MVRI[i].PlayerOwner.PlayerReplicationInfo.PlayerID == PlayerID)
        {
            bFound = true;
            VictumID = i;
            PlayerName = MVRI[i].PlayerOwner.PlayerReplicationInfo.PlayerName;
            break;
        }
    }
    if(!bFound)
        return;

    if( MVRI[VoterID].KickVote == VictumID ) // if vote is for same player stop
        return;

    if( PlayerController(Voter).PlayerReplicationInfo.bAdmin )  // Administrator Vote
    {
        log("___Admin " $ PlayerController(Voter).PlayerReplicationInfo.PlayerName $ " kicked " $ PlayerName,'MapVote');
        if (GS != none)
            GS.LogKickVote(Controller(Voter).PlayerReplicationInfo,VictumID);
        KickPlayer(VictumID);
        return;
    }

    if( PlayerController(Voter).PlayerReplicationInfo.bOnlySpectator )
    {
        // Spectators cant vote
        PlayerController(Voter).ClientMessage(lmsgSpectatorsCantVote);
        return;
    }

    // cant kick admin
    if(MVRI[VictumID].PlayerOwner.PlayerReplicationInfo.bAdmin || NetConnection(MVRI[VictumID].PlayerOwner.Player) == None)
    {
        TextMessage = lmsgKickVoteAdmin;
        TextMessage = repl(TextMessage,"%playername%",PlayerController(Voter).PlayerReplicationInfo.PlayerName);
        Level.Game.Broadcast(self,TextMessage);
        return;
    }

    log("___" $ PlayerController(Voter).PlayerReplicationInfo.PlayerName $ " placed a kick vote against " $ PlayerName,'MapVote');
    if(bAnonymousKicking)
    {
        TextMessage = lmsgAnonymousKickVote;
        TextMessage = repl(TextMessage,"%playername%",PlayerName);
        Level.Game.Broadcast(self,TextMessage);
    }
    else
    {
        TextMessage = lmsgKickVote;
        TextMessage = repl(TextMessage,"%playername1%",PlayerController(Voter).PlayerReplicationInfo.PlayerName);
        TextMessage = repl(TextMessage,"%playername2%",PlayerName);
        Level.Game.Broadcast(self,TextMessage);
    }

    if (GS != none && !bAnonymousKicking)
        GS.LogKickVote(Controller(Voter).PlayerReplicationInfo,VictumID);
    else if (GS != none && bAnonymousKicking)
        GS.LogKickVote(none,VictumID);

    PreviousVote = MVRI[VoterID].KickVote;
    MVRI[VoterID].KickVote = VictumID;

    UpdateKickVoteCount(MVRI[VictumID].PlayerID, 1);
    if( PreviousVote > -1 )
        UpdateKickVoteCount(MVRI[PreviousVote].PlayerID, -1); // undo previous vote

    TallyKickVotes();
}

function KickPlayer(int PlayerIndex)
{
    local int i;

    if( MVRI[PlayerIndex] == none || MVRI[PlayerIndex].PlayerOwner == none )
        return;

    TextMessage = "%playername% has been kicked.";
    TextMessage = repl(TextMessage,"%playername%",MVRI[PlayerIndex].PlayerOwner.PlayerReplicationInfo.PlayerName);
    Level.Game.Broadcast(self,TextMessage);

    if(bKickVote)
    {
        // Reset votes
        for(i=0;i < MVRI.Length;i++)
        {
            if(MVRI[i] != None && MVRI[i].KickVote != -1)
                MVRI[i].KickVote = -1;
        }
    }

    //close his/her voting window if open
    if(MVRI[PlayerIndex] != None)
        MVRI[PlayerIndex].CloseWindow();

    log("___" $ MVRI[PlayerIndex].PlayerOwner.PlayerReplicationInfo.PlayerName $ " has been kicked.",'MapVote');
    Level.Game.AccessControl.BanPlayer(MVRI[PlayerIndex].PlayerOwner, True); // session type ban

    if(GS != none)
        GS.GameEvent("kickvote",string(PlayerIndex),none);
}

function TallyVotes(bool bForceMapSwitch)
{
    local int        index,x,y,topmap,r,mapidx,gameidx;
    local array<int> VoteCount;
    local array<int> Ranking;
    local int        PlayersThatVoted;
    local int        TieCount;
    local string     CurrentMap;
    local int        Votes;
    local MapHistoryInfo MapInfo;

    if(bLevelSwitchPending)
        return;

    PlayersThatVoted = 0;
    VoteCount.Length = GameConfig.Length * MapCount;
    // note: VoteCount array is a 2 dimension array VoteCount[GameConfigIndex, MapIndex]
    //       Maps ->
    //       0 1 2 3 4 5 6 7 8
    // G     - - - - - - - - -
    // a  0 |0 0 0 0 0 0 0 2 0
    // m  1 |0 0 0 2 0 0 0 0 0
    // e  2 |0 6 0 0 0 5 0 0 0
    // s  3 |0 0 0 3 0 0 0 0 0

    for(x=0;x < MVRI.Length;x++) // for each player
    {
        if(MVRI[x] != none && MVRI[x].MapVote > -1 && MVRI[x].GameVote > -1) // if this player has voted
        {
            PlayersThatVoted++;

            if(bScoreMode)
            {
                if(bAccumulationMode)
                    Votes = GetAccVote(MVRI[x].PlayerOwner) + int(GetPlayerScore(MVRI[x].PlayerOwner));
                else
                    Votes = int(GetPlayerScore(MVRI[x].PlayerOwner));
            }
            else
            {  // Not Score Mode == Majority (one vote per player)
                if(bAccumulationMode)
                    Votes = GetAccVote(MVRI[x].PlayerOwner) + 1;
                else
                    Votes = 1;
            }
            VoteCount[MVRI[x].GameVote * MapCount + MVRI[x].MapVote] = VoteCount[MVRI[x].GameVote * MapCount + MVRI[x].MapVote] + Votes;

            if(!bScoreMode)
            {
                // If more then half the players voted for the same map as this player then force a winner
                if(Level.Game.NumPlayers > 2 && float(VoteCount[MVRI[x].GameVote * MapCount + MVRI[x].MapVote]) / float(Level.Game.NumPlayers) > 0.5 && Level.Game.bGameEnded)
                    bForceMapSwitch = true;
            }
        }
    }
    log("___Voted - " $ PlayersThatVoted,'MapVoteDebug');

    if(Level.Game.NumPlayers > 2 && !Level.Game.bGameEnded && !bMidGameVote && (float(PlayersThatVoted) / float(Level.Game.NumPlayers)) * 100 >= MidGameVotePercent) // Mid game vote initiated
    {
        Level.Game.Broadcast(self,lmsgMidGameVote);
        bMidGameVote = true;
        // Start voting count-down timer
        TimeLeft = VoteTimeLimit;
        ScoreBoardTime = 1;
        settimer(1,true);
    }

    index = 0;
    for(x=0;x < VoteCount.Length;x++) // for each map
    {
        if(VoteCount[x] > 0)
        {
            Ranking.Insert(index,1);
            Ranking[index++] = x; // copy all vote indexes to the ranking list if someone has voted for it.
        }
    }

    if(PlayersThatVoted > 1)
    {
        // bubble sort ranking list by vote count
        for(x=0; x<index-1; x++)
        {
            for(y=x+1; y<index; y++)
            {
                if(VoteCount[Ranking[x]] < VoteCount[Ranking[y]])
                {
                topmap = Ranking[x];
                Ranking[x] = Ranking[y];
                Ranking[y] = topmap;
                }
            }
        }
    }
    else
    {
        if(PlayersThatVoted == 0)
        {
            GetDefaultMap(mapidx, gameidx);
            topmap = gameidx * MapCount + mapidx;
        }
        else
            topmap = Ranking[0];  // only one player voted
    }

    //Check for a tie
    if(PlayersThatVoted > 1) // need more than one player vote for a tie
    {
        if(index > 1 && VoteCount[Ranking[0]] == VoteCount[Ranking[1]] && VoteCount[Ranking[0]] != 0)
        {
            TieCount = 1;
            for(x=1; x<index; x++)
            {
                if(VoteCount[Ranking[0]] == VoteCount[Ranking[x]])
                TieCount++;
            }
            //reminder ---> int Rand( int Max ); Returns a random number from 0 to Max-1.
            topmap = Ranking[Rand(TieCount)];

            // Don't allow same map to be choosen
            CurrentMap = GetURLMap();

            r = 0;
            while(MapList[topmap - (topmap/MapCount) * MapCount].MapName ~= CurrentMap)
            {
                topmap = Ranking[Rand(TieCount)];
                if(r++>100)
                    break;  // just incase
            }
        }
        else
        {
            topmap = Ranking[0];
        }
    }

    // if everyone has voted go ahead and change map
    if(bForceMapSwitch || (Level.Game.NumPlayers == PlayersThatVoted && Level.Game.NumPlayers > 0) )
    {
        if(MapList[topmap - topmap/MapCount * MapCount].MapName == "")
            return;

        TextMessage = lmsgMapWon;
        TextMessage = repl(TextMessage,"%mapname%",MapList[topmap - topmap/MapCount * MapCount].MapName $ "(" $ GameConfig[topmap/MapCount].Acronym $ ")");
        Level.Game.Broadcast(self,TextMessage);

        if (GS != none)
            GS.GameEvent("mapvote",MapList[topmap - topmap/MapCount * MapCount].MapName,none);

        CloseAllVoteWindows();

        MapInfo = History.PlayMap(MapList[topmap - topmap/MapCount * MapCount].MapName);

        ServerTravelString = SetupGameMap(MapList[topmap - topmap/MapCount * MapCount], topmap/MapCount, MapInfo);
        log("ServerTravelString = " $ ServerTravelString ,'MapVoteDebug');

        History.Save();

        if(bEliminationMode)
            RepeatLimit++;

        if(bAccumulationMode)
            SaveAccVotes(topmap - topmap/MapCount * MapCount, topmap/MapCount);

        //if(bEliminationMode || bAccumulationMode)
        CurrentGameConfig = topmap/MapCount;
        SaveConfig();

        bLevelSwitchPending = true;
        settimer(Level.TimeDilation,true);  // timer() will monitor the server-travel and detect a failure

        Level.ServerTravel(ServerTravelString, false);    // change the map
    }
}

DefaultProperties
{

}
